@@ -1,4 +1,4 @@
-// Copyright 2017 Istio Authors
+// Copyright 2017 Fortio Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -106,6 +106,7 @@ var (
 	goMaxProcsFlag  = flag.Int("gomaxprocs", 0, "Setting for runtime.GOMAXPROCS, <1 doesn't change the default")
 	profileFlag     = flag.String("profile", "", "write .cpu and .mem profiles to `file`")
 	grpcFlag        = flag.Bool("grpc", false, "Use GRPC (health check by default, add -ping for ping) for load testing")
+	grpcCompression = flag.Bool("grpc-compression", false, "Enable grpc compression")
 	echoPortFlag    = flag.String("http-port", "8080",
 		"http echo server port. Can be in the form of host:port, ip:port, `port` or /unix/domain/path or \""+disabled+"\".")
 	tcpPortFlag = flag.String("tcp-port", "8078",
@@ -178,6 +179,7 @@ var (
 	accessLogFileFormat = flag.String("access-log-format", "json",
 		"`format` for access log. Supported values: [json, influx]")
 	calcQPS = flag.Bool("calc-qps", false, "Calculate the qps based on number of requests (-n) and duration (-t)")
+	pprofOn = flag.Bool("pprof", false, "Enable pprof http endpoint in the Web UI handler server")
 )
 
 // serverArgCheck always returns true after checking arguments length.
@@ -207,10 +209,6 @@ func FortioMain(hook bincommon.FortioHook) {
 	scli.ServerMain() // will Exit if there were arguments/flags errors.
 
 	fnet.ChangeMaxPayloadSize(*newMaxPayloadSizeKb * fnet.KILOBYTE)
-	percList, err := stats.ParsePercentiles(*percentilesFlag)
-	if err != nil {
-		cli.ErrUsage("Unable to extract percentiles from -p: %v", err)
-	}
 	baseURL := strings.Trim(*baseURLFlag, " \t\n\r/") // remove trailing slash and other whitespace
 	sync := strings.TrimSpace(*syncFlag)
 	if sync != "" {
@@ -221,11 +219,13 @@ func FortioMain(hook bincommon.FortioHook) {
 	isServer := false
 	switch cli.Command {
 	case "curl":
+		log.SetDefaultsForClientTools()
 		fortioLoad(true, nil, hook)
 	case "nc":
+		log.SetDefaultsForClientTools()
 		fortioNC()
 	case "load":
-		fortioLoad(*curlFlag, percList, hook)
+		fortioLoad(*curlFlag, percList(), hook)
 	case "redirect":
 		isServer = serverArgCheck()
 		fhttp.RedirectToHTTPS(*redirectFlag)
@@ -252,6 +252,7 @@ func FortioMain(hook bincommon.FortioHook) {
 		}
 	case "server":
 		isServer = serverArgCheck()
+		tlsOptions := &bincommon.SharedHTTPOptions().TLSOptions
 		if *tcpPortFlag != disabled {
 			fnet.TCPEchoServer("tcp-echo", *tcpPortFlag)
 		}
@@ -259,18 +260,29 @@ func FortioMain(hook bincommon.FortioHook) {
 			fnet.UDPEchoServer("udp-echo", *udpPortFlag, *udpAsyncFlag)
 		}
 		if *grpcPortFlag != disabled {
-			fgrpc.PingServer(*grpcPortFlag, *healthSvcFlag, uint32(*maxStreamsFlag), &bincommon.SharedHTTPOptions().TLSOptions)
+			fgrpc.PingServer(*grpcPortFlag, *healthSvcFlag, uint32(*maxStreamsFlag), tlsOptions)
 		}
 		if *redirectFlag != disabled {
 			fhttp.RedirectToHTTPS(*redirectFlag)
 		}
 		if *echoPortFlag != disabled {
-			if !ui.Serve(hook, baseURL, *echoPortFlag, *echoDbgPathFlag, *uiPathFlag, *dataDirFlag, percList) {
+			uiCfg := ui.ServerConfig{
+				BaseURL:        baseURL,
+				Port:           *echoPortFlag,
+				DebugPath:      *echoDbgPathFlag,
+				UIPath:         *uiPathFlag,
+				DataDir:        *dataDirFlag,
+				PProfOn:        *pprofOn,
+				PercentileList: percList(),
+				TLSOptions:     tlsOptions,
+			}
+			if !ui.Serve(hook, &uiCfg) {
 				os.Exit(1) // error already logged
 			}
 		}
 		startProxies()
 	case "grpcping":
+		log.SetDefaultsForClientTools()
 		grpcClient()
 	default:
 		cli.ErrUsage("Error: unknown command %q", cli.Command)
@@ -280,6 +292,14 @@ func FortioMain(hook bincommon.FortioHook) {
 	}
 }
 
+func percList() (percList []float64) {
+	percList, err := stats.ParsePercentiles(*percentilesFlag)
+	if err != nil {
+		cli.ErrUsage("Unable to extract percentiles from -p: %v", err)
+	}
+	return
+}
+
 func serverLoop(sync string) {
 	// To get a start time log/timestamp in the logs
 	log.Infof("All fortio %s servers started!", version.Long())
@@ -430,6 +450,8 @@ func fortioLoad(justCurl bool, percList []float64, hook bincommon.FortioHook) {
 			Delay:              *pingDelayFlag,
 			UsePing:            *doPingLoadFlag,
 			Metadata:           httpHeader2grpcMetadata(httpOpts.AllHeaders()),
+			GrpcCompression:    *grpcCompression,
+			Profiler:           *profileFlag,
 		}
 		o.TLSOptions = httpOpts.TLSOptions
 		res, err = fgrpc.RunGRPCTest(&o)
