@@ -1,4 +1,4 @@
-// Copyright 2017 Istio Authors
+// Copyright 2017 Fortio Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -37,13 +37,21 @@ import (
 	"fortio.org/fortio/stats"
 	"fortio.org/log"
 	"github.com/google/uuid"
+	"golang.org/x/net/http2"
 )
 
 // Fetcher is the Url content fetcher that the different client implements.
 type Fetcher interface {
 	// Fetch returns http code, data, offset of body (for client which returns
 	// headers)
+	// Deprecated: use StreamFetch with a DataWriter (nil if you don't need the data) instead.
 	Fetch(context.Context) (int, []byte, int)
+	// StreamFetch returns http code and body bytes read
+	// (body is streamed to Dest writer or null),and header size for the fast client.
+	StreamFetch(context.Context) (int, int64, uint)
+	// HasBuffer is true for the fast client and false for golang standard library based client.
+	// it's used to know if calling Fetch() is actually better (fast client with headers to stderr)
+	HasBuffer() bool
 	// Close() cleans up connections and state - must be paired with NewClient calls.
 	Close()
 	// GetIPAddress() returns the occurrence of ip address used by this client connection,
@@ -141,6 +149,10 @@ func (h *HTTPOptions) URLSchemeCheck() {
 		log.Errf("Unexpected init with empty url")
 		return
 	}
+	if h.PayloadReader != nil && !h.H2 {
+		log.Infof("PayloadReader set, switching to H2")
+		h.H2 = true
+	}
 	if h.H2 && !h.DisableFastClient {
 		log.Infof("H2 requested, switching to std client")
 		h.DisableFastClient = true
@@ -161,7 +173,7 @@ func (h *HTTPOptions) URLSchemeCheck() {
 }
 
 const (
-	retcodeOffset = len("HTTP/1.X ")
+	retcodeOffset = int64(len("HTTP/1.X "))
 	// HTTPReqTimeOutDefaultValue is the default timeout value. 3s.
 	HTTPReqTimeOutDefaultValue = 3 * time.Second
 )
@@ -189,8 +201,10 @@ type HTTPOptions struct {
 	UserCredentials  string        // user credentials for authorization
 	ContentType      string        // indicates request body type, implies POST instead of GET
 	Payload          []byte        // body for http request, implies POST if not empty.
+	MethodOverride   string        // optional http method override. Otherwise GET or POST when a payload or ContentType is set.
 	LogErrors        bool          // whether to log non 2xx code as they occur or not
 	ID               int           `json:"-"` // thread/connect id to use for logging (thread id when used as a runner)
+	UniqueID         int64         `json:"-"` // Run identifier when used through a runner, copied from RunnerOptions.RunID
 	SequentialWarmup bool          // whether to do http(s) runs warmup sequentially or in parallel (new default is //)
 	ConnReuseRange   [2]int        // range of max number of connection to reuse for each thread.
 	// When false, re-resolve the DNS name when the connection breaks.
@@ -204,6 +218,9 @@ type HTTPOptions struct {
 	// Optional Transport chain factory to use if set. Only effective when using std client.
 	// pass otelhttp.NewTransport for instance.
 	Transport CreateTransport `json:"-"`
+	// These following 2 options are only making sense for single operation (curl) mode.
+	PayloadReader io.Reader `json:"-"` // if set, Payload is ignored and this is used instead.
+	DataWriter    io.Writer `json:"-"` // if set, the response body is written to this writer.
 }
 
 type CreateClientTrace func(ctx context.Context) *httptrace.ClientTrace
@@ -271,6 +288,9 @@ func (h *HTTPOptions) AllHeaders() http.Header {
 
 // Method returns the method of the http req.
 func (h *HTTPOptions) Method() string {
+	if len(h.MethodOverride) > 0 {
+		return h.MethodOverride
+	}
 	if len(h.Payload) > 0 || h.ContentType != "" {
 		return fnet.POST
 	}
@@ -292,18 +312,23 @@ func (h *HTTPOptions) AddAndValidateExtraHeader(hdr string) error {
 	// No TrimSpace for the value, so we can set empty "" vs just whitespace " " which
 	// will get trimmed later but treated differently: not emitted vs emitted empty for User-Agent.
 	value := s[1]
+	// 2 headers need trimmed to not have extra spaces:
+	trimmedValue := strings.TrimSpace(value)
 	switch strings.ToLower(key) {
 	case "host":
-		log.LogVf("Will be setting special Host header to %s", value)
-		h.hostOverride = strings.TrimSpace(value) // This one needs to be trimmed
+		log.LogVf("Will be setting special Host header to %s", trimmedValue)
+		h.hostOverride = trimmedValue // This one needs to be trimmed
 	case "user-agent":
 		if value == "" {
 			log.Infof("Deleting default User-Agent: header.")
 			h.extraHeaders.Del(key)
 		} else {
-			log.Infof("User-Agent being Set to %q", value)
+			log.Infof("User-Agent being set to %q", value)
 			h.extraHeaders.Set(key, value)
 		}
+	case "content-type":
+		log.LogVf("Content-Type being set to %q", trimmedValue)
+		h.ContentType = trimmedValue
 	default:
 		log.LogVf("Setting regular extra header %s: %s", key, value)
 		h.extraHeaders.Add(key, value)
@@ -346,14 +371,33 @@ func (h *HTTPOptions) ValidateAndSetConnectionReuseRange(inp string) error {
 // newHttpRequest makes a new http GET request for url with User-Agent.
 func newHTTPRequest(o *HTTPOptions) (*http.Request, error) {
 	method := o.Method()
+	log.Debugf("newHTTPRequest %s %s", method, o.URL)
 	var body io.Reader
-	if method == fnet.POST {
+	if o.PayloadReader != nil {
+		body = o.PayloadReader
+	} else if len(o.Payload) > 0 || method == fnet.POST {
 		body = bytes.NewReader(o.Payload)
 	}
 	//nolint:noctx // we pass context later in Run()/Fetch()
 	req, err := http.NewRequest(method, o.URL, body)
+	if err == nil { //nolint:nestif
+		// Additional validation for the URL so we abort early on fatal errors even for the std client.
+		// fixes #784
+		if req.URL == nil || req.URL.Host == "" {
+			err = fmt.Errorf("invalid url '%s'", o.URL)
+		} else {
+			host := req.URL.Hostname()
+			if o.Resolve != "" {
+				host = o.Resolve
+			}
+			log.Debugf("Std client extra validation - host part of url (or resolve option) is %q", host)
+			_, err = fnet.ResolveAll(context.Background(), host, "ip")
+		}
+	}
 	if err != nil {
-		log.Errf("[%d] Unable to make %s request for %s : %v", o.ID, method, o.URL, err)
+		log.S(log.Error, "Unable to make request",
+			log.Attr("method", method), log.Attr("url", o.URL), log.Attr("err", err),
+			log.Attr("thread", o.ID), log.Attr("run", o.UniqueID))
 		return nil, err
 	}
 	req.Header = o.GenerateHeaders()
@@ -369,7 +413,8 @@ func newHTTPRequest(o *HTTPOptions) (*http.Request, error) {
 	}
 	bytes, err := httputil.DumpRequestOut(req, false)
 	if err != nil {
-		log.Errf("[%d] Unable to dump request: %v", o.ID, err)
+		log.S(log.Error, "Unable to dump request",
+			log.Attr("err", err), log.Attr("thread", o.ID), log.Attr("run", o.UniqueID))
 	} else {
 		log.Debugf("[%d] For URL %s, sending:\n%s", o.ID, o.URL, bytes)
 	}
@@ -386,15 +431,21 @@ type Client struct {
 	body                 []byte // original body of the request
 	req                  *http.Request
 	client               *http.Client
-	transport            *http.Transport
+	transport            Transport
 	pathContainsUUID     bool // if url contains the "{uuid}" pattern (lowercase)
 	rawQueryContainsUUID bool // if any query params contains the "{uuid}" pattern (lowercase)
 	bodyContainsUUID     bool // if body contains the "{uuid}" pattern (lowercase)
 	logErrors            bool
 	id                   int
+	runID                int64
 	ipAddrUsage          *stats.Occurrence
 	connectStats         *stats.Histogram
 	clientTrace          CreateClientTrace
+	dataWriter           io.Writer
+}
+
+func (c *Client) HasBuffer() bool {
+	return false
 }
 
 // Close cleans up any resources used by NewStdClient.
@@ -403,7 +454,7 @@ func (c *Client) Close() {
 	if c.req != nil {
 		if c.req.Body != nil {
 			if err := c.req.Body.Close(); err != nil {
-				log.Warnf("[%d] Error closing std client body: %v", c.id, err)
+				log.S(log.Warning, "Error closing std client body", log.Attr("err", err), log.Attr("thread", c.id), log.Attr("run", c.runID))
 			}
 		}
 		c.req = nil
@@ -420,8 +471,21 @@ func (c *Client) ChangeURL(urlStr string) (err error) {
 	return err
 }
 
-// Fetch fetches the byte and code for pre created client.
+// Fetch is the backward compatible version of StreamFetch to avoid
+// rewriting a bunch of tests but should not be used for load tests
+// (where we don't care about the data and only the stats)
+// Deprecated: use StreamFetch instead.
 func (c *Client) Fetch(ctx context.Context) (int, []byte, int) {
+	var buf bytes.Buffer
+	c.dataWriter = &buf
+	status, _, _ := c.StreamFetch(ctx)
+	return status, buf.Bytes(), 0
+}
+
+// StreamFetch fetches the byte and code for pre created std client.
+// header length (3rd returned value) is always 0 for that client
+// and only available with the fastclient.
+func (c *Client) StreamFetch(ctx context.Context) (int, int64, uint) {
 	// req can't be null (client itself would be null in that case)
 	var req *http.Request
 	if c.clientTrace != nil {
@@ -457,34 +521,41 @@ func (c *Client) Fetch(ctx context.Context) (int, []byte, int) {
 	}
 	resp, err := c.client.Do(req)
 	if err != nil {
-		log.Errf("[%d] Unable to send %s request for %s : %v", c.id, req.Method, c.url, err)
-		return -1, []byte(err.Error()), 0
+		log.S(log.Error, "Unable to send request",
+			log.Attr("method", req.Method), log.Attr("url", c.url), log.Attr("err", err),
+			log.Attr("thread", c.id), log.Attr("run", c.runID))
+		return -1, -1, 0
 	}
 	var data []byte
 	if log.LogDebug() {
 		if data, err = httputil.DumpResponse(resp, false); err != nil {
-			log.Errf("[%d] Unable to dump response %v", c.id, err)
+			log.S(log.Error, "Unable to dump response", log.Attr("err", err), log.Attr("thread", c.id), log.Attr("run", c.runID))
 		} else {
 			log.Debugf("[%d] For URL %s, received:\n%s", c.id, c.url, data)
 		}
 	}
-	data, err = io.ReadAll(resp.Body)
+	if c.dataWriter == nil {
+		c.dataWriter = io.Discard
+	}
+	var n int64
+	n, err = io.Copy(c.dataWriter, resp.Body)
 	resp.Body.Close()
 	if err != nil {
-		log.Errf("[%d] Unable to read response for %s : %v", c.id, c.url, err)
+		log.S(log.Error, "Unable to read response",
+			log.Attr("err", err), log.Attr("thread", c.id), log.Attr("run", c.runID))
 		code := resp.StatusCode
 		if codeIsOK(code) {
 			code = http.StatusNoContent
-			log.Warnf("[%d] Ok code despite read error, switching code to %d", c.id, code)
+			log.S(log.Warning, "Ok code despite read error, switching code to 204", log.Attr("thread", c.id), log.Attr("run", c.runID))
 		}
-		return code, data, 0
+		return code, n, 0
 	}
 	code := resp.StatusCode
 	log.Debugf("[%d] Got %d : %s for %s %s - response is %d bytes", c.id, code, resp.Status, req.Method, c.url, len(data))
 	if c.logErrors && !codeIsOK(code) {
-		log.Warnf("[%d] Non ok http code %d", c.id, code)
+		log.S(log.Warning, "Non ok http code", log.Attr("code", code), log.Attr("thread", c.id), log.Attr("run", c.runID))
 	}
-	return code, data, 0
+	return code, n, 0
 }
 
 // GetIPAddress get the ip address that DNS resolves to when using stdClient and connection stats.
@@ -502,6 +573,12 @@ func NewClient(o *HTTPOptions) (Fetcher, error) {
 	return NewFastClient(o)
 }
 
+// Transport common interface between http.Transport and http2.Transport.
+type Transport interface {
+	http.RoundTripper
+	CloseIdleConnections()
+}
+
 // NewStdClient creates a client object that wraps the net/http standard client.
 func NewStdClient(o *HTTPOptions) (*Client, error) {
 	o.Init(o.URL) // also normalizes NumConnections etc to be valid.
@@ -527,52 +604,69 @@ func NewStdClient(o *HTTPOptions) (*Client, error) {
 		// Keep track of timing for connection (re)establishment.
 		connectStats: stats.NewHistogram(o.Offset.Seconds(), o.Resolution),
 		clientTrace:  o.ClientTrace,
+		dataWriter:   o.DataWriter,
+		runID:        o.UniqueID,
 	}
-
-	tr := http.Transport{
+	dialCtx := func(ctx context.Context, network, addr string) (net.Conn, error) {
+		// redirect all connections to resolved ip, and use cn as sni host
+		if o.Resolve != "" {
+			addr = o.Resolve + addr[strings.LastIndex(addr, ":"):]
+		}
+		var conn net.Conn
+		now := time.Now()
+		conn, err = (&net.Dialer{
+			Timeout: o.HTTPReqTimeOut,
+		}).DialContext(ctx, network, addr)
+		client.connectStats.Record(time.Since(now).Seconds())
+		if conn != nil {
+			newRemoteAddress := conn.RemoteAddr().String()
+			// No change when it wasn't set before (first time) and when the value isn't actually changing either.
+			if req.RemoteAddr != "" && newRemoteAddress != req.RemoteAddr {
+				log.S(log.Info, "Standard client IP address changed", log.Str("dest", req.RemoteAddr), log.Str("new_ip", newRemoteAddress),
+					log.Attr("thread", client.id), log.Attr("run", client.runID))
+			}
+			req.RemoteAddr = newRemoteAddress
+			client.ipAddrUsage.Record(req.RemoteAddr)
+		}
+		return conn, err
+	}
+	tr := &http.Transport{
 		MaxIdleConns:        o.NumConnections,
 		MaxIdleConnsPerHost: o.NumConnections,
 		DisableCompression:  !o.Compression,
 		DisableKeepAlives:   o.DisableKeepAlive,
 		Proxy:               http.ProxyFromEnvironment,
-		DialContext: func(ctx context.Context, network, addr string) (net.Conn, error) {
-			// redirect all connections to resolved ip, and use cn as sni host
-			if o.Resolve != "" {
-				addr = o.Resolve + addr[strings.LastIndex(addr, ":"):]
-			}
-			var conn net.Conn
-			now := time.Now()
-			conn, err = (&net.Dialer{
-				Timeout: o.HTTPReqTimeOut,
-			}).DialContext(ctx, network, addr)
-			client.connectStats.Record(time.Since(now).Seconds())
-			if conn != nil {
-				newRemoteAddress := conn.RemoteAddr().String()
-				// No change when it wasn't set before (first time) and when the value isn't actually changing either.
-				if req.RemoteAddr != "" && newRemoteAddress != req.RemoteAddr {
-					log.Infof("[%d] Standard client IP address changed from %s to %s", client.id, req.RemoteAddr, newRemoteAddress)
-				}
-				req.RemoteAddr = newRemoteAddress
-				client.ipAddrUsage.Record(req.RemoteAddr)
-			}
-			return conn, err
-		},
+		DialContext:         dialCtx,
 		TLSHandshakeTimeout: o.HTTPReqTimeOut,
 		ForceAttemptHTTP2:   o.H2,
 	}
-	var rt http.RoundTripper = &tr
-	if o.Transport != nil {
-		rt = o.Transport(rt)
-	}
-	client.client.Transport = rt
-	client.transport = &tr // internal transport, unwrapped (to close idle conns)
-
+	client.transport = tr // internal transport, unwrapped (to close idle conns)
 	if o.https {
 		tr.TLSClientConfig, err = o.TLSOptions.TLSConfig()
 		if err != nil {
 			return nil, err
 		}
+	} else if o.H2 {
+		// Need to do h2c instead of normal transport
+		// Note: this likely means connection multiplexing / not sure how to force unique connections
+		// with http2.Transport.
+		if err != nil {
+			return nil, err
+		}
+		tr2 := &http2.Transport{
+			AllowHTTP: true,
+			DialTLSContext: func(ctx context.Context, network, addr string, cfg *tls.Config) (net.Conn, error) {
+				return dialCtx(ctx, network, addr)
+			},
+			DisableCompression: !o.Compression,
+		}
+		client.transport = tr2
 	}
+	var rt http.RoundTripper = client.transport
+	if o.Transport != nil {
+		rt = o.Transport(rt)
+	}
+	client.client.Transport = rt
 	if !o.FollowRedirects {
 		// Lets us see the raw response instead of auto following redirects.
 		client.client.CheckRedirect = func(req *http.Request, via []*http.Request) error {
@@ -585,21 +679,38 @@ func NewStdClient(o *HTTPOptions) (*Client, error) {
 // FetchURL fetches the data at the given url using the standard client and default options.
 // Returns the http status code (http.StatusOK == 200 for success) and the data.
 // To be used only for single fetches or when performance doesn't matter as the client is closed at the end.
+// Deprecated: use StreamURL instead.
 func FetchURL(url string) (int, []byte) {
+	var w bytes.Buffer
+	code := StreamURL(url, &w)
+	return code, w.Bytes()
+}
+
+func StreamURL(url string, w io.Writer) int {
 	o := NewHTTPOptions(url)
 	// Maximize chances of getting the data back, vs the raw payload like the fast client
 	o.DisableFastClient = true
 	o.FollowRedirects = true
-	return Fetch(o)
+	o.DataWriter = w
+	return StreamFetch(o)
 }
 
 // Fetch creates a client an performs a fetch according to the http options passed in.
 // To be used only for single fetches or when performance doesn't matter as the client is closed at the end.
+// Deprecated: use StreamFetch instead.
 func Fetch(httpOptions *HTTPOptions) (int, []byte) {
+	var w bytes.Buffer
+	httpOptions.DataWriter = &w
+	return StreamFetch(httpOptions), w.Bytes()
+}
+
+// Fetch creates a client an performs a fetch according to the http options passed in.
+// To be used only for single fetches or when performance doesn't matter as the client is closed at the end.
+func StreamFetch(httpOptions *HTTPOptions) int {
 	cli, _ := NewClient(httpOptions)
-	code, data, _ := cli.Fetch(context.Background())
+	code, _, _ := cli.StreamFetch(context.Background())
 	cli.Close()
-	return code, data
+	return code
 }
 
 // FastClient is a fast, lockfree single purpose http 1.0/1.1 client.
@@ -609,10 +720,10 @@ type FastClient struct {
 	dest         net.Addr
 	socket       net.Conn
 	socketCount  int // number of sockets attempts, same as the new connectStats.Count() + DNS errors if any.
-	size         int
+	size         int64
 	code         int
 	errorCount   int
-	headerLen    int
+	headerLen    uint
 	url          string
 	host         string
 	hostname     string
@@ -625,6 +736,7 @@ type FastClient struct {
 	uuidMarkers  [][]byte
 	logErrors    bool
 	id           int
+	runID        int64
 	https        bool
 	tlsConfig    *tls.Config
 	// Resolve the DNS name for each connection
@@ -636,6 +748,7 @@ type FastClient struct {
 	connReuse      int
 	reuseCount     int
 	connectStats   *stats.Histogram
+	dataWriter     io.Writer
 }
 
 // GetIPAddress get ip address that DNS resolved to when using fast client and connection stats.
@@ -643,12 +756,17 @@ func (c *FastClient) GetIPAddress() (*stats.Occurrence, *stats.Histogram) {
 	return c.ipAddrUsage, c.connectStats
 }
 
+func (c *FastClient) HasBuffer() bool {
+	return true
+}
+
 // Close cleans up any resources used by FastClient.
 func (c *FastClient) Close() {
 	log.Debugf("[%d] Closing %p %s socket count %d", c.id, c, c.url, c.socketCount)
 	if c.socket != nil {
 		if err := c.socket.Close(); err != nil {
-			log.Warnf("[%d] Error closing fast client's socket: %v", c.id, err)
+			log.S(log.Warning, "Error closing fast client's socket",
+				log.Attr("err", err), log.Attr("thread", c.id), log.Attr("run", c.runID))
 		}
 		c.socket = nil
 	}
@@ -659,6 +777,7 @@ func (c *FastClient) Close() {
 // the beginning and then reused many times.
 func NewFastClient(o *HTTPOptions) (Fetcher, error) { //nolint:funlen
 	method := o.Method()
+	log.Debugf("NewFastClient %s %s", method, o.URL)
 	payloadLen := len(o.Payload)
 	o.Init(o.URL)
 	proto := "1.1"
@@ -685,7 +804,8 @@ func NewFastClient(o *HTTPOptions) (Fetcher, error) { //nolint:funlen
 	// Parse the url, extract components.
 	url, err := url.Parse(urlString)
 	if err != nil {
-		log.Errf("[%d] Bad url %q : %v", o.ID, urlString, err)
+		log.S(log.Error, "Bad url", log.Str("url", urlString), log.Attr("err", err),
+			log.Attr("thread", o.ID), log.Attr("run", o.UniqueID))
 		return nil, err
 	}
 
@@ -702,11 +822,12 @@ func NewFastClient(o *HTTPOptions) (Fetcher, error) { //nolint:funlen
 	// note: Host includes the port
 	bc := FastClient{
 		url: o.URL, host: url.Host, hostname: url.Hostname(), port: url.Port(),
-		http10: o.HTTP10, halfClose: o.AllowHalfClose, logErrors: o.LogErrors, id: o.ID,
+		http10: o.HTTP10, halfClose: o.AllowHalfClose, logErrors: o.LogErrors, id: o.ID, runID: o.UniqueID,
 		https: o.https, connReuseRange: o.ConnReuseRange, connReuse: connReuse,
 		resolve: o.Resolve, noResolveEachConn: o.NoResolveEachConn, ipAddrUsage: stats.NewOccurrence(),
 		// Keep track of timing for connection (re)establishment.
 		connectStats: stats.NewHistogram(o.Offset.Seconds(), o.Resolution),
+		dataWriter:   o.DataWriter,
 	}
 	if o.https {
 		bc.tlsConfig, err = o.TLSOptions.TLSConfig()
@@ -721,7 +842,8 @@ func NewFastClient(o *HTTPOptions) (Fetcher, error) { //nolint:funlen
 	}
 	var addr net.Addr
 	if o.UnixDomainSocket != "" {
-		log.Infof("[%d] Using unix domain socket %v instead of %v %v", bc.id, o.UnixDomainSocket, bc.hostname, bc.port)
+		log.S(log.Info, "Using unix domain socket", log.Attr("path", o.UnixDomainSocket),
+			log.Attr("thread", bc.id), log.Attr("run", bc.runID))
 		uds := &net.UnixAddr{Name: o.UnixDomainSocket, Net: fnet.UnixDomainSocket}
 		addr = uds
 	} else {
@@ -780,8 +902,11 @@ func NewFastClient(o *HTTPOptions) (Fetcher, error) { //nolint:funlen
 }
 
 // return the result from the state.
-func (c *FastClient) returnRes() (int, []byte, int) {
-	return c.code, c.buffer[:c.size], c.headerLen
+func (c *FastClient) returnRes() (int, int64, uint) {
+	if c.dataWriter != nil && c.dataWriter != io.Discard {
+		_, _ = c.dataWriter.Write(c.buffer[:c.size])
+	}
+	return c.code, c.size, c.headerLen
 }
 
 // connect to destination.
@@ -795,7 +920,8 @@ func (c *FastClient) connect(ctx context.Context) net.Conn {
 		c.dest, err = resolve(ctx, c.hostname, c.port, c.resolve, c.ipAddrUsage)
 		log.Debugf("[%d] Hostname %v resolve to ip %v", c.id, c.hostname, c.dest)
 		if err != nil {
-			log.Errf("[%d] Unable to resolve hostname %v: %v", c.id, c.hostname, err)
+			log.S(log.Error, "Unable to resolve hostname", log.Str("hostname", c.hostname), log.Attr("err", err),
+				log.Attr("thread", c.id), log.Attr("run", c.runID))
 			return nil
 		}
 	}
@@ -806,14 +932,16 @@ func (c *FastClient) connect(ctx context.Context) net.Conn {
 		socket, err = tls.DialWithDialer(d, c.dest.Network(), c.dest.String(), c.tlsConfig)
 		c.connectStats.Record(time.Since(now).Seconds())
 		if err != nil {
-			log.Errf("[%d] Unable to TLS connect to %v : %v", c.id, c.dest, err)
+			log.S(log.Error, "Unable to TLS connect", log.Attr("dest", c.dest), log.Attr("err", err),
+				log.Attr("thread", c.id), log.Attr("run", c.runID))
 			return nil
 		}
 	} else {
 		socket, err = d.Dial(c.dest.Network(), c.dest.String())
 		c.connectStats.Record(time.Since(now).Seconds())
 		if err != nil {
-			log.Errf("[%d] Unable to connect to %v : %v", c.id, c.dest, err)
+			log.S(log.Error, "Unable to connect", log.Attr("dest", c.dest), log.Attr("err", err),
+				log.Attr("thread", c.id), log.Attr("run", c.runID))
 			return nil
 		}
 	}
@@ -831,6 +959,16 @@ const (
 
 // Fetch fetches the url content. Returns http code, data, offset of body.
 func (c *FastClient) Fetch(ctx context.Context) (int, []byte, int) {
+	// We don't want to even use a writer as the buffer is there and fixed
+	// so we keep that path optimized.
+	c.dataWriter = nil
+	// we're inlining the old returnRes() below so no need to capture the return values
+	code, _, _ := c.StreamFetch(ctx)
+	return code, c.buffer[:c.size], int(c.headerLen)
+}
+
+// Fetch fetches the url content. Returns http code, data written to the writer, length of headers.
+func (c *FastClient) StreamFetch(ctx context.Context) (int, int64, uint) {
 	c.code = SocketError
 	c.size = 0
 	c.headerLen = 0
@@ -866,35 +1004,37 @@ func (c *FastClient) Fetch(ctx context.Context) (int, []byte, int) {
 	if err != nil || conErr != nil {
 		if canReuse {
 			// it's ok for the (idle) socket to die once, auto reconnect:
-			log.Infof("[%d] Closing dead socket %v (%v)", c.id, c.dest, err)
+			log.S(log.Info, "Closing dead socket", log.Attr("err", err), log.Attr("thread", c.id), log.Attr("run", c.runID))
 			conn.Close()
 			c.errorCount++
-			return c.Fetch(ctx) // recurse once
+			return c.StreamFetch(ctx) // recurse once
 		}
-		log.Errf("[%d] Unable to write to %v : %v", c.id, c.dest, err)
+		log.S(log.Error, "Unable to write", log.Attr("err", err), log.Attr("thread", c.id), log.Attr("run", c.runID))
 		return c.returnRes()
 	}
 	if n != len(c.req) {
-		log.Errf("[%d] Short write to %v : %d instead of %d", c.id, c.dest, n, len(c.req))
+		log.S(log.Error, "Short write", log.Attr("err", err), log.Attr("actual", n), log.Attr("expected", len(c.req)),
+			log.Attr("thread", c.id), log.Attr("run", c.runID))
 		return c.returnRes()
 	}
 	if !c.keepAlive && c.halfClose { //nolint:nestif
 		tcpConn, ok := conn.(*net.TCPConn)
 		if ok {
 			if err = tcpConn.CloseWrite(); err != nil {
-				log.Errf("[%d] Unable to close write to %v : %v", c.id, c.dest, err)
+				log.S(log.Error, "Unable to close write", log.Attr("err", err), log.Attr("thread", c.id), log.Attr("run", c.runID))
 				return c.returnRes()
 			} // else:
 			log.Debugf("[%d] Half closed ok after sending request %v", c.id, c.dest)
 		} else {
-			log.Warnf("[%d] Unable to close write non tcp connection %v", c.id, c.dest)
+			log.S(log.Warning, "Unable to close write non tcp connection",
+				log.Attr("err", err), log.Attr("thread", c.id), log.Attr("run", c.runID))
 		}
 	}
 	// Read the response:
 	c.readResponse(conn, canReuse)
 	if c.code == RetryOnce {
 		// Special "eof on reused socket" code
-		return c.Fetch(ctx) // recurse once
+		return c.StreamFetch(ctx) // recurse once
 	}
 	// Return the result:
 	return c.returnRes()
@@ -909,7 +1049,7 @@ func codeIsOK(code int) bool {
 //
 //nolint:nestif,funlen,gocognit,gocyclo,maintidx // TODO: refactor - unwiedly/ugly atm.
 func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
-	max := len(c.buffer)
+	max := int64(len(c.buffer))
 	parsedHeaders := false
 	// TODO: safer to start with -1 / SocketError and fix ok for http 1.0
 	c.code = http.StatusOK // In http 1.0 mode we don't bother parsing anything
@@ -922,15 +1062,17 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 		// Ugly way to cover the case where we get more than 1 chunk at the end
 		// TODO: need automated tests
 		if !skipRead {
-			n, err := conn.Read(c.buffer[c.size:])
+			nI, err := conn.Read(c.buffer[c.size:])
+			n := int64(nI)
 			if err != nil {
 				if reusedSocket && c.size == 0 {
 					// Ok for reused socket to be dead once (close by server)
-					log.Infof("[%d] Closing dead socket %v (err %v at first read)", c.id, c.dest, err)
+					log.S(log.Info, "Closing dead socket (err at first read)",
+						log.Attr("err", err), log.Attr("thread", c.id), log.Attr("run", c.runID))
 					c.errorCount++
 					err = conn.Close() // close the previous one
 					if err != nil {
-						log.Warnf("[%d] Error closing dead socket %v: %v", c.id, c.dest, err)
+						log.S(log.Warning, "Error closing dead socket", log.Attr("err", err), log.Attr("thread", c.id), log.Attr("run", c.runID))
 					}
 					c.code = RetryOnce // special "retry once" code
 					return
@@ -939,14 +1081,15 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 					// handled below as possibly normal end of stream after we read something
 					break
 				}
-				log.Errf("[%d] Read error for %v %d : %v", c.id, c.dest, c.size, err)
+				log.S(log.Error, "Read error", log.Attr("err", err), log.Attr("size", c.size), log.Attr("dest", c.dest), log.Str("url", c.url),
+					log.Attr("thread", c.id), log.Attr("run", c.runID))
 				c.code = SocketError
 				break
 			}
 			c.size += n
 			if log.LogDebug() {
 				log.Debugf("[%d] Read ok %d total %d so far (-%d headers = %d data) %s",
-					c.id, n, c.size, c.headerLen, c.size-c.headerLen, DebugSummary(c.buffer[c.size-n:c.size], 256))
+					c.id, n, c.size, c.headerLen, c.size-int64(c.headerLen), DebugSummary(c.buffer[c.size-n:c.size], 256))
 			}
 		}
 		skipRead = false
@@ -954,11 +1097,12 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 		// at least parse the http retcode:
 		if !parsedHeaders && c.parseHeaders && c.size >= retcodeOffset+3 {
 			// even if the bytes are garbage we'll get a non 200 code (bytes are unsigned)
-			c.code = ParseDecimal(c.buffer[retcodeOffset : retcodeOffset+3]) // TODO do that only once...
+			c.code = int(ParseDecimal(c.buffer[retcodeOffset : retcodeOffset+3])) // TODO do that only once...
 			// TODO handle 100 Continue, make the "ok" codes configurable
 			if !codeIsOK(c.code) {
 				if c.logErrors {
-					log.Warnf("[%d] Non ok http code %d (%v)", c.id, c.code, string(c.buffer[:retcodeOffset+3]))
+					log.S(log.Warning, "Non ok http code", log.Attr("code", c.code), log.Str("status", string(c.buffer[:retcodeOffset+3])),
+						log.Attr("thread", c.id), log.Attr("run", c.runID))
 				}
 				break
 			}
@@ -970,11 +1114,11 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 			idx := endofHeadersStart
 			for idx < c.size-1 {
 				if c.buffer[idx] == '\r' && c.buffer[idx+1] == '\n' {
-					if c.headerLen == idx-2 { // found end of headers
+					if int64(c.headerLen) == idx-2 { // found end of headers
 						parsedHeaders = true
 						break
 					}
-					c.headerLen = idx
+					c.headerLen = uint(idx)
 					idx++
 				}
 				idx++
@@ -988,17 +1132,18 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 				}
 				// Find the content length or chunked mode
 				if keepAlive {
-					var contentLength int
+					var contentLength int64
 					found, offset := FoldFind(c.buffer[:c.headerLen], contentLengthHeader)
 					if found {
 						// Content-Length mode:
 						contentLength = ParseDecimal(c.buffer[offset+len(contentLengthHeader) : c.headerLen])
 						if contentLength < 0 {
-							log.Warnf("[%d] Warning: content-length unparsable %s", c.id, string(c.buffer[offset+2:offset+len(contentLengthHeader)+4]))
+							log.S(log.Warning, "Content-length unparsable", log.Str("cl", string(c.buffer[offset+2:offset+len(contentLengthHeader)+4])),
+								log.Attr("thread", c.id), log.Attr("run", c.runID))
 							keepAlive = false
 							break
 						}
-						max = c.headerLen + contentLength
+						max = int64(c.headerLen) + contentLength
 						if log.LogDebug() { // somehow without the if we spend 400ms/10s in LogV (!)
 							log.Debugf("[%d] found content length %d", c.id, contentLength)
 						}
@@ -1006,39 +1151,44 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 						// Chunked mode (or err/missing):
 						if found, _ := FoldFind(c.buffer[:c.headerLen], chunkedHeader); found {
 							chunkedMode = true
-							var dataStart int
+							var dataStart int64
 							dataStart, contentLength = ParseChunkSize(c.buffer[c.headerLen:c.size])
 							if contentLength == -1 {
 								// chunk length not available yet
 								log.LogVf("[%d] chunk mode but no first chunk length yet, reading more", c.id)
-								max = c.headerLen
+								max = int64(c.headerLen)
 								continue
 							}
-							max = c.headerLen + dataStart + contentLength + 2 // extra CR LF
+							max = int64(c.headerLen) + dataStart + contentLength + 2 // extra CR LF
 							log.Debugf("[%d] chunk-length is %d (%s) setting max to %d",
-								c.id, contentLength, c.buffer[c.headerLen:c.headerLen+dataStart-2],
+								c.id, contentLength, c.buffer[c.headerLen:int64(c.headerLen)+dataStart-2],
 								max)
 						} else {
 							if log.LogVerbose() {
 								log.LogVf("[%d] Warning: content-length missing in %q", c.id, string(c.buffer[:c.headerLen]))
 							} else {
-								log.Warnf("[%d] Warning: content-length missing (%d bytes headers)", c.id, c.headerLen)
+								log.S(log.Warning, "Content-length missing",
+									log.Attr("header_len", c.headerLen), log.Attr("thread", c.id), log.Attr("run", c.runID))
 							}
 							keepAlive = false // can't keep keepAlive
 							break
 						}
 					} // end of content-length section
-					if max > len(c.buffer) {
-						log.Warnf("[%d] Buffer is too small for headers %d + data %d - change -httpbufferkb flag to at least %d",
-							c.id, c.headerLen, contentLength, (c.headerLen+contentLength)/1024+1)
+					if max > int64(len(c.buffer)) {
+						log.S(log.Warning, "Buffer is too small for headers + data - change -httpbufferkb flag",
+							log.Attr("header_len", c.headerLen),
+							log.Attr("content_length", contentLength),
+							log.Attr("buffer_needed", (int64(c.headerLen)+contentLength)/1024+1),
+							log.Attr("thread", c.id), log.Attr("run", c.runID))
 						// TODO: just consume the extra instead
-						max = len(c.buffer)
+						// or rather use the dataWriter post headers
+						max = int64(len(c.buffer))
 					}
 					if checkConnectionClosedHeader {
 						if found, _ := FoldFind(c.buffer[:c.headerLen], connectionCloseHeader); found {
-							log.Infof("[%d] Server wants to close connection, no keep-alive!", c.id)
+							log.S(log.Info, "Server wants to close connection, no keep-alive!", log.Attr("thread", c.id), log.Attr("run", c.runID))
 							keepAlive = false
-							max = len(c.buffer) // reset to read as much as available
+							max = int64(len(c.buffer)) // reset to read as much as available
 						}
 					}
 				}
@@ -1046,10 +1196,12 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 		} // end of big if parse header
 		if c.size >= max {
 			if !keepAlive {
-				log.Errf("[%d] More data is available but stopping after %d, increase -httpbufferkb", c.id, max)
+				log.S(log.Error, "More data is available but stopping after max, increase -httpbufferkb",
+					log.Attr("max", max), log.Attr("thread", c.id), log.Attr("run", c.runID))
 			}
 			if !parsedHeaders && c.parseHeaders {
-				log.Errf("[%d] Buffer too small (%d) to even finish reading headers, increase -httpbufferkb to get all the data", c.id, max)
+				log.S(log.Error, "Buffer too small to even finish reading headers, increase -httpbufferkb to get all the data",
+					log.Attr("max", max), log.Attr("thread", c.id), log.Attr("run", c.runID))
 				keepAlive = false
 			}
 			if chunkedMode {
@@ -1059,20 +1211,24 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 					if c.size == max {
 						log.Debugf("[%d] Couldn't find next chunk size, reading more %d %d", c.id, max, c.size)
 					} else {
-						log.Infof("[%d] Partial chunk size (%s), reading more %d %d", c.id, DebugSummary(c.buffer[max:c.size], 20), max, c.size)
+						log.S(log.Info, "Partial chunk size, reading more",
+							log.Str("buf", DebugSummary(c.buffer[max:c.size], 20)), log.Attr("max", max), log.Attr("size", c.size),
+							log.Attr("thread", c.id), log.Attr("run", c.runID))
 					}
 					continue
 				} else if nextChunkLen == 0 {
 					log.Debugf("[%d] Found last chunk %d %d", c.id, max+dataStart, c.size)
 					if c.size != max+dataStart+2 || string(c.buffer[c.size-2:c.size]) != "\r\n" {
-						log.Errf("[%d] Unexpected mismatch at the end sz=%d expected %d; end of buffer %q",
-							c.id, c.size, max+dataStart+2, c.buffer[max:c.size])
+						log.S(log.Error, "Unexpected mismatch at the end",
+							log.Attr("size", c.size), log.Attr("expected", max+dataStart+2),
+							log.Attr("end-of_buffer", c.buffer[max:c.size]),
+							log.Attr("thread", c.id), log.Attr("run", c.runID))
 					}
 				} else {
 					max += dataStart + nextChunkLen + 2 // extra CR LF
 					log.Debugf("[%d] One more chunk %d -> new max %d", c.id, nextChunkLen, max)
-					if max > len(c.buffer) {
-						log.Errf("[%d] Buffer too small for %d data", c.id, max)
+					if max > int64(len(c.buffer)) {
+						log.S(log.Error, "Buffer too small for data", log.Attr("size", max), log.Attr("thread", c.id), log.Attr("run", c.runID))
 					} else {
 						if max <= c.size {
 							log.Debugf("[%d] Enough data to reach next chunk, skipping a read", c.id)
@@ -1090,7 +1246,8 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 		c.socket = conn // keep the open socket
 	} else {
 		if err := conn.Close(); err != nil {
-			log.Errf("[%d] Close error %v %d : %v", c.id, c.dest, c.size, err)
+			log.S(log.Error, "Close error", log.Attr("err", err), log.Attr("size", c.size),
+				log.Attr("thread", c.id), log.Attr("run", c.runID))
 		} else {
 			log.Debugf("[%d] Closed ok from %v after reading %d bytes", c.id, c.dest, c.size)
 		}
