@@ -1,4 +1,4 @@
-// Copyright 2017 Istio Authors
+// Copyright 2017 Fortio Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -262,7 +262,7 @@ func TestASCIIToUpper(t *testing.T) {
 func TestParseDecimal(t *testing.T) {
 	tests := []struct {
 		input    string // input
-		expected int    // output
+		expected int64  // output
 	}{
 		{"", -1},
 		{"3", 3},
@@ -283,8 +283,8 @@ func TestParseDecimal(t *testing.T) {
 func TestParseChunkSize(t *testing.T) {
 	tests := []struct {
 		input     string // input
-		expOffset int    // expected offset
-		expValue  int    // expected value
+		expOffset int64  // expected offset
+		expValue  int64  // expected value
 	}{
 		// Errors :
 		{"", 0, -1},
@@ -720,15 +720,20 @@ func TestBadURLStdClient(t *testing.T) {
 	opts := NewHTTPOptions("not a valid url")
 	cli, err := NewStdClient(opts)
 	if cli != nil || err == nil {
-		t.Errorf("config1: got a client %v despite bogus url %s", cli, opts.URL)
+		t.Errorf("config1: got a client %v despite bogus url %q", cli, opts.URL)
 		cli.Close()
 	}
-
 	opts.URL = "http://doesnotexist.fortio.org"
-	cli, _ = NewStdClient(opts)
-	code, _, _ := cli.Fetch(context.Background())
-	if code != -1 {
-		t.Errorf("config2: client can send request despite bogus url %s", opts.URL)
+	cli, err = NewStdClient(opts)
+	if cli != nil || err == nil {
+		t.Errorf("config2: got a client %v despite bogus host in url %q", cli, opts.URL)
+		cli.Close()
+	}
+	opts.URL = ""
+	cli, err = NewStdClient(opts)
+	if cli != nil || err == nil {
+		t.Errorf("config3: got a client %v despite empty url %q", cli, opts.URL)
+		cli.Close()
 	}
 }
 
@@ -759,7 +764,9 @@ func TestDefaultPort(t *testing.T) {
 	if cli == nil {
 		t.Fatalf("Couldn't get a client using NewClient on modified opts: %v", err)
 	}
-	// currently fast client fails with https:
+	if !cli.HasBuffer() {
+		t.Errorf("didn't get expected fast client with buffer")
+	}
 	code, _, _ = cli.Fetch(context.Background())
 	if code != 200 {
 		t.Errorf("Standard client http error code %d", code)
@@ -861,16 +868,32 @@ func TestPayloadForClient(t *testing.T) {
 			nil,
 			"GET",
 		},
+		{
+			"",
+			[]byte{},
+			"GET",
+		},
+		{
+			"Foo", // once we set a content-type we get a POST instead of GET (alternative to -X)
+			nil,
+			"POST",
+		},
 	}
 	for _, test := range tests {
 		hOptions := HTTPOptions{}
 		hOptions.URL = "www.google.com"
-		hOptions.ContentType = test.contentType
+		err := hOptions.AddAndValidateExtraHeader("conTENT-tYPE: " + test.contentType)
+		if err != nil {
+			t.Errorf("Got error %v adding header", err)
+		}
+		if hOptions.ContentType != test.contentType {
+			t.Errorf("Got %q, expected %q as a content type from header addition", hOptions.ContentType, test.contentType)
+		}
 		hOptions.Payload = test.payload
 		client, _ := NewStdClient(&hOptions)
 		contentType := client.req.Header.Get("Content-Type")
 		if contentType != test.contentType {
-			t.Errorf("Got %s, expected %s as a content type", contentType, test.contentType)
+			t.Errorf("Got %q, expected %q as a content type", contentType, test.contentType)
 		}
 		method := client.req.Method
 		if method != test.expectedMethod {
@@ -883,7 +906,7 @@ func TestPayloadForClient(t *testing.T) {
 			}
 			continue
 		}
-		buf := new(bytes.Buffer)
+		var buf bytes.Buffer // A Buffer needs no initialization. https://pkg.go.dev/bytes#Buffer
 		buf.ReadFrom(body)
 		payload := buf.Bytes()
 		if !bytes.Equal(payload, test.payload) {
@@ -892,6 +915,47 @@ func TestPayloadForClient(t *testing.T) {
 	}
 }
 
+func TestMethodOverride(t *testing.T) {
+	_, addr := ServeTCP("0", "/debug")
+	o := HTTPOptions{URL: fmt.Sprintf("http://localhost:%d/debug", addr.Port), MethodOverride: "FOO"}
+	for _, fastClient := range []bool{true, false} {
+		o.DisableFastClient = !fastClient
+		cli, _ := NewClient(&o)
+		ctx := context.Background()
+		code, data, _ := cli.Fetch(ctx)
+		if code != 200 {
+			t.Errorf("Non ok code %d for debug override method %q", code, o.MethodOverride)
+		}
+		expected := []byte("FOO /debug HTTP/1.1")
+		if !bytes.Contains(data, expected) {
+			t.Errorf("Method not echoed back in fastClient %v client %s (expecting %s)", fastClient, DebugSummary(data, 512), expected)
+		}
+	}
+}
+
+func TestPostFromContentType(t *testing.T) {
+	_, addr := ServeTCP("0", "/debug")
+	o := HTTPOptions{URL: fmt.Sprintf("http://localhost:%d/debug", addr.Port)}
+	o.AddAndValidateExtraHeader("content-type: foo/bar; xyz")
+	for _, fastClient := range []bool{true, false} {
+		o.DisableFastClient = !fastClient
+		cli, _ := NewClient(&o)
+		ctx := context.Background()
+		code, data, _ := cli.Fetch(ctx)
+		if code != 200 {
+			t.Errorf("Non ok code %d for debug override method %q", code, o.MethodOverride)
+		}
+		expected := []byte("POST /debug HTTP/1.1")
+		if !bytes.Contains(data, expected) {
+			t.Errorf("POST not found in fastClient %v client %s (expecting %s)", fastClient, DebugSummary(data, 512), expected)
+		}
+		expected = []byte("Content-Type: foo/bar; xyz")
+		if !bytes.Contains(data, expected) {
+			t.Errorf("Content-type not found in fastClient %v client %s (expecting %s)", fastClient, DebugSummary(data, 512), expected)
+		}
+	}
+}
+
 func TestPayloadForFastClient(t *testing.T) {
 	tests := []struct {
 		contentType     string
@@ -1262,7 +1326,7 @@ func TestFetch2(t *testing.T) {
 	}
 }
 
-func TestFetch2Header(t *testing.T) {
+func TestFetch2IncomingHeader(t *testing.T) {
 	mux, addr := ServeTCP("0", "")
 	mux.HandleFunc("/fetch2/", FetcherHandler2)
 	url := fmt.Sprintf("localhost:%d/fetch2/?url=http://localhost:%d/echo%%3fheader%%3dFoo:Bar", addr.Port, addr.Port)
@@ -1275,6 +1339,34 @@ func TestFetch2Header(t *testing.T) {
 	}
 }
 
+func TestFetch2OutgoingHeaders(t *testing.T) {
+	mux, addr := ServeTCP("0", "/debug")
+	mux.HandleFunc("/fetch2/", FetcherHandler2)
+	// added &H= and &H=incomplete to cover these 2 error cases
+	url := fmt.Sprintf("localhost:%d/fetch2/?url=http://localhost:%d/debug"+
+		"&H=foo:Bar&payload=a-test&H=&H=IncompleteHeader&H=Content-TYPE:foo/bar42",
+		addr.Port, addr.Port)
+	code, data := Fetch(&HTTPOptions{URL: url, Payload: []byte("a-longer-different-payload")})
+	if code != http.StatusOK {
+		t.Errorf("Got %d %s instead of ok for %s", code, DebugSummary(data, 256), url)
+	}
+	if !bytes.Contains(data, []byte("Foo: Bar")) {
+		t.Errorf("Result %s doesn't contain expected Foo: Bar header", DebugSummary(data, 1024))
+	}
+	if !bytes.Contains(data, []byte("POST /debug HTTP/1.1")) {
+		t.Errorf("Passing payload should mean POST - got %s", DebugSummary(data, 1024))
+	}
+	if !bytes.Contains(data, []byte("body:\n\na-test\n")) {
+		t.Errorf("Passing payload be echoed - got %s", DebugSummary(data, 1024))
+	}
+	if !bytes.Contains(data, []byte("Content-Length: 6\n")) {
+		t.Errorf("Payload length should be the right one - got %s", DebugSummary(data, 1024))
+	}
+	if !bytes.Contains(data, []byte("Content-Type: foo/bar42\n")) {
+		t.Errorf("Payload length should be the right one - got %s", DebugSummary(data, 1024))
+	}
+}
+
 func TestFetch2Errors(t *testing.T) {
 	mux, addr := ServeTCP("0", "")
 	mux.HandleFunc("/fetch2/", FetcherHandler2)
