@@ -1,4 +1,4 @@
-// Copyright 2017 Istio Authors
+// Copyright 2017 Fortio Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -161,8 +161,8 @@ func FoldFind(haystack []byte, needle []byte) (bool, int) {
 // ParseDecimal extracts the first positive integer number from the input.
 // spaces are ignored.
 // any character that isn't a digit cause the parsing to stop.
-func ParseDecimal(inp []byte) int {
-	res := -1
+func ParseDecimal(inp []byte) int64 {
+	res := int64(-1)
 	for _, b := range inp {
 		if b == ' ' && res == -1 {
 			continue
@@ -170,7 +170,7 @@ func ParseDecimal(inp []byte) int {
 		if b < '0' || b > '9' {
 			break
 		}
-		digit := int(b - '0')
+		digit := int64(b - '0')
 		if res == -1 {
 			res = digit
 		} else {
@@ -183,13 +183,13 @@ func ParseDecimal(inp []byte) int {
 // ParseChunkSize extracts the chunk size and consumes the line.
 // Returns the offset of the data and the size of the chunk,
 // 0, -1 when not found.
-func ParseChunkSize(inp []byte) (int, int) {
+func ParseChunkSize(inp []byte) (int64, int64) {
 	if log.LogDebug() {
 		log.Debugf("ParseChunkSize(%s)", DebugSummary(inp, 128))
 	}
-	res := -1
-	off := 0
-	end := len(inp)
+	res := int64(-1)
+	off := int64(0)
+	end := int64(len(inp))
 	inDigits := true
 	for {
 		if off >= end {
@@ -197,11 +197,11 @@ func ParseChunkSize(inp []byte) (int, int) {
 		}
 		if inDigits { //nolint:nestif
 			b := toUpper(inp[off])
-			var digit int
+			var digit int64
 			if b >= 'A' && b <= 'F' {
-				digit = 10 + int(b-'A')
+				digit = 10 + int64(b-'A')
 			} else if b >= '0' && b <= '9' {
-				digit = int(b - '0')
+				digit = int64(b - '0')
 			} else {
 				inDigits = false
 				if res == -1 {
@@ -629,3 +629,40 @@ func (r *SyncReader) Read(p []byte) (n int, err error) {
 
 	return r.reader.Read(p)
 }
+
+// CommonHTTPOptionsFromForm is used essentially in ui/uihandler.go but we want to reuse some options
+// for fetching URLs too. Also ideally would get refactored to work for rapi/.
+func CommonHTTPOptionsFromForm(r *http.Request) *HTTPOptions {
+	url := r.FormValue("url")
+	payload := r.FormValue("payload")
+	methodOverride := r.FormValue("X")
+	logErrors := (r.FormValue("log-errors") == "on")
+	h2 := (r.FormValue("h2") == "on")
+	httpsInsecure := (r.FormValue("https-insecure") == "on")
+	resolve := r.FormValue("resolve")
+	timeoutStr := strings.TrimSpace(r.FormValue("timeout"))
+	timeout, _ := time.ParseDuration(timeoutStr) // will be 0 if empty, which is handled by runner and opts
+	httpopts := &HTTPOptions{}
+	// to be normalized in init 0 replaced by default value only in http runner, not here as this could be a tcp or udp runner
+	httpopts.URL = url // fixes #651 - ie don't normalize here
+	httpopts.HTTPReqTimeOut = timeout
+	httpopts.Insecure = httpsInsecure
+	httpopts.Resolve = resolve
+	httpopts.H2 = h2
+	httpopts.LogErrors = logErrors
+	httpopts.MethodOverride = methodOverride
+	if len(payload) > 0 {
+		httpopts.Payload = []byte(payload)
+	}
+	for _, header := range r.Form["H"] {
+		if len(header) == 0 {
+			continue
+		}
+		log.LogVf("adding header %v", header)
+		err := httpopts.AddAndValidateExtraHeader(header)
+		if err != nil {
+			log.Errf("Error adding custom headers: %v", err)
+		}
+	}
+	return httpopts
+}
