@@ -1,4 +1,4 @@
-// Copyright 2017 Istio Authors.
+// Copyright 2017 Fortio Authors.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -79,7 +79,15 @@ func TestHTTPRunner(t *testing.T) {
 	}
 	o1 = rawOpts
 	o1.URL = "http://www.doesnotexist.badtld/"
-	c, _ := NewStdClient(&o1)
+	c, err := NewStdClient(&o1)
+	if err == nil || c != nil {
+		t.Errorf("Std Client bad host should error early")
+	}
+	o1.URL = "http://debug.fortio.org" // should resolve fine
+	c, err = NewStdClient(&o1)
+	if err != nil {
+		t.Errorf("Unexpected error once url is fixed: %v", err)
+	}
 	c.ChangeURL(rawOpts.URL)
 	if r, _, _ := c.Fetch(context.Background()); r != http.StatusOK {
 		t.Errorf("Std Client with raw option should still work with warning in logs")
@@ -263,7 +271,6 @@ func gUnzipData(t *testing.T, data []byte) (resData []byte) {
 	return
 }
 
-//nolint:gocognit
 func TestAccessLogAndTrace(t *testing.T) {
 	mux, addr := DynamicHTTPServer(false)
 	mux.HandleFunc("/echo-for-alog/", EchoHandler)
@@ -283,6 +290,7 @@ func TestAccessLogAndTrace(t *testing.T) {
 	traceFactory := func(ctx context.Context) *httptrace.ClientTrace { return trace }
 	opts.DisableFastClient = true
 	opts.ClientTrace = traceFactory
+	opts.LogErrors = true
 	for _, format := range []string{"json", "influx"} {
 		dir := t.TempDir()
 		fname := path.Join(dir, "access.log")
@@ -638,3 +646,33 @@ func getIPUsageCount(ipCountMap map[string]int) (count int) {
 
 	return count
 }
+
+func TestRunnerErrors(t *testing.T) {
+	mux, addr := DynamicHTTPServer(false)
+	mux.HandleFunc("/", EchoHandler)
+	baseURL := fmt.Sprintf("http://localhost:%d/", addr.Port)
+
+	opts := HTTPRunnerOptions{}
+	opts.QPS = 2
+	opts.Duration = 1 * time.Second
+	opts.URL = baseURL + "?status=555"
+	opts.SequentialWarmup = true
+	_, err := RunHTTPTest(&opts)
+	t.Logf("Got expected error %v", err)
+	if err == nil {
+		t.Error("Expecting an because of not allowing initial errors")
+	}
+	log.SetLogLevel(log.Verbose)
+	opts.AllowInitialErrors = true
+	opts.NumThreads = 1
+	opts.LogErrors = true
+	_, err = RunHTTPTest(&opts)
+	if err != nil {
+		t.Errorf("Expecting no error because of allowing initial errors, got: %v", err)
+	}
+	opts.URL = "http:// not a url / foo"
+	_, err = RunHTTPTest(&opts)
+	if err == nil {
+		t.Error("Expecting an error because of invalid url")
+	}
+}
