@@ -1,4 +1,4 @@
-// Copyright 2017 Istio Authors
+// Copyright 2017 Fortio Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -34,6 +34,7 @@ import (
 	"sync"
 	"time"
 
+	"fortio.org/fortio/jrpc"
 	"fortio.org/fortio/stats"
 	"fortio.org/fortio/version"
 	"fortio.org/log"
@@ -90,6 +91,7 @@ type Aborter struct {
 	stopRequested bool
 }
 
+// Note this can cause data race if called without holding the lock. TODO: maybe use reentrant lock. but this is for debug only.
 func (a *Aborter) String() string {
 	return fmt.Sprintf("{Aborter %p stopChan %v startChan %v hasStarted %v stopRequested %v}",
 		a, a.StopChan, a.StartChan, a.hasStarted, a.stopRequested)
@@ -110,34 +112,53 @@ func (a *Aborter) Abort(wait bool) {
 	a.stopRequested = true
 	started := a.hasStarted
 	if started || !wait {
-		log.LogVf("ABORT Closing %v", a)
+		log.LogVf("ABORT Closing already started or not waiting %v", a)
 		close(a.StopChan)
 		a.StopChan = nil
 		a.Unlock()
 		if started {
 			log.LogVf("ABORT reading start channel")
-			// shouldn't block/hang, just purging/resetting
-			<-a.StartChan
+			// shouldn't block/hang, just purging/resetting - but another aborter (line 137 might have consumed it already)
+			select {
+			case b := <-a.StartChan:
+				log.LogVf("ABORT done reading start channel, got %v", b)
+			default:
+				log.LogVf("ABORT start channel empty (not quite expected)")
+			}
 			a.Lock()
 			a.hasStarted = false
 			a.Unlock()
 		}
 		return
 	}
+	// Wait & not started case:
 	a.Unlock()
 	log.LogVf("ABORT Waiting for start")
-	<-a.StartChan
-	log.LogVf("ABORT Done waiting for start")
+	b := <-a.StartChan
+	log.LogVf("ABORT Done waiting for start, got %v", b)
 	a.Lock()
-	a.hasStarted = false
 	if a.StopChan != nil {
-		log.LogVf("ABORT Closing %+v", a)
+		log.LogVf("ABORT Closing wasn't started %+v", a)
 		close(a.StopChan)
 		a.StopChan = nil
 	}
+	a.hasStarted = false
 	a.Unlock()
 }
 
+// RecordStart records the start of the run. (used by httprunner in error cases to fake start so rapi stop and wait can work).
+func (a *Aborter) RecordStart() (chan struct{}, bool) {
+	a.Lock()
+	a.hasStarted = true
+	startedChan := a.StartChan
+	runnerChan := a.StopChan // need a copy to not race with assignment to nil
+	shouldAbort := a.stopRequested
+	log.LogVf("RUNNER starting... can now be Abort()ed, telling %v - %v", a, startedChan)
+	a.Unlock()
+	startedChan <- true
+	return runnerChan, shouldAbort
+}
+
 // Reset returns the aborter to original state, for (unit test) reuse.
 // Note that it doesn't recreate the closed stop chan.
 func (a *Aborter) Reset() {
@@ -239,6 +260,8 @@ type RunnerResults struct {
 	AccessLoggerInfo        string
 	// Same as RunnerOptions ID:  Unique 96 character ID used as reference to saved json file. Created during Normalize().
 	ID string
+	// If the run doesn't even start because of for instance an invalid host name, this will be set (all omitted on success)
+	jrpc.ServerReply
 }
 
 // HasRunnerResult is the interface implictly implemented by HTTPRunnerResults
@@ -363,7 +386,7 @@ func (r *RunnerOptions) Normalize() {
 // to nil under lock so it can be called multiple times and not create panic for
 // already closed channel.
 func (r *RunnerOptions) Abort() {
-	log.LogVf("Abort called for %p %+v", r, r)
+	log.LogVf("Abort called for %p", r)
 	if r.Stop != nil {
 		r.Stop.Abort(false)
 	}
@@ -404,7 +427,7 @@ func (r *periodicRunner) runQPSSetup(extra string) (requestedDuration string, re
 		// Always print that as we need ^C to interrupt, in that case the user need to notice
 		_, _ = fmt.Fprintf(r.Out, "Starting at %g qps with %d thread(s) [gomax %d] until interrupted%s\n",
 			r.QPS, r.NumThreads, runtime.GOMAXPROCS(0), extra)
-		return
+		return //nolint:nakedret // it's fine/cleaner to not repeat all the parameters we just set/we return.
 	}
 	// else:
 	requestedDuration = fmt.Sprint(r.Duration)
@@ -473,14 +496,7 @@ func (r *periodicRunner) runMaxQPSSetup(extra string) (requestedDuration string,
 // Run starts the runner.
 func (r *periodicRunner) Run() RunnerResults {
 	aborter := r.Stop
-	aborter.Lock()
-	runnerChan := aborter.StopChan // need a copy to not race with assignment to nil
-	startedChan := aborter.StartChan
-	aborter.hasStarted = true
-	shouldAbort := aborter.stopRequested
-	aborter.Unlock()
-	log.LogVf("RUNNER starting... can now be Abort()ed, telling %v - %v", aborter, startedChan)
-	startedChan <- true
+	runnerChan, shouldAbort := aborter.RecordStart()
 	useQPS := (r.QPS > 0)
 	// r.Exactly is > 0 if we use Exactly iterations instead of the duration.
 	useExactly := (r.Exactly > 0)
@@ -524,6 +540,7 @@ func (r *periodicRunner) Run() RunnerResults {
 			0, 0, r.NumThreads, version.Short(), functionDuration.Export().CalcPercentiles(r.Percentiles),
 			errorsDuration.Export().CalcPercentiles(r.Percentiles),
 			r.Exactly, r.Jitter, r.Uniform, r.NoCatchUp, r.RunID, loggerInfo, r.ID,
+			*jrpc.NewErrorReply("Aborted before even starting", nil),
 		}
 	}
 	if r.NumThreads <= 1 {
@@ -561,6 +578,8 @@ func (r *periodicRunner) Run() RunnerResults {
 	actualQPS := float64(functionDuration.Count) / elapsed.Seconds()
 	if log.Log(log.Warning) {
 		_, _ = fmt.Fprintf(r.Out, "Ended after %v : %d calls. qps=%.5g\n", elapsed, functionDuration.Count, actualQPS)
+		log.S(log.Info, "Run ended", log.Attr("run", r.RunID), log.Attr("elapsed", elapsed),
+			log.Attr("calls", functionDuration.Count), log.Attr("qps", actualQPS))
 	}
 	if useQPS { //nolint:nestif
 		percentNegative := 100. * float64(sleepTime.Hdata[0]) / float64(sleepTime.Count)
@@ -587,6 +606,7 @@ func (r *periodicRunner) Run() RunnerResults {
 		actualQPS, elapsed, r.NumThreads, version.Short(), functionDuration.Export().CalcPercentiles(r.Percentiles),
 		errorsDuration.Export().CalcPercentiles(r.Percentiles),
 		r.Exactly, r.Jitter, r.Uniform, r.NoCatchUp, r.RunID, loggerInfo, r.ID,
+		jrpc.ServerReply{Error: false},
 	}
 	if log.Log(log.Warning) {
 		result.DurationHistogram.Print(r.Out, "Aggregated Function Time")
@@ -688,11 +708,12 @@ func NewFileAccessLoggerByType(filePath string, accessType AccessLoggerType) (Ac
 
 // Before each Run().
 func (a *fileAccessLogger) Start(ctx context.Context, threadID ThreadID, iter int64, startTime time.Time) context.Context {
+	log.Debugf("fileAccessLogger start thread %d iter %d, %v", threadID, iter, startTime)
 	return ctx
 }
 
 // Report logs a single request to a file.
-func (a *fileAccessLogger) Report(ctx context.Context, thread ThreadID, iter int64, time time.Time,
+func (a *fileAccessLogger) Report(_ context.Context, thread ThreadID, iter int64, time time.Time,
 	latency float64, status bool, details string,
 ) {
 	a.mu.Lock()
