@@ -30,6 +30,7 @@ import (
 	"fortio.org/fortio/bincommon"
 	"fortio.org/fortio/fgrpc"
 	"fortio.org/fortio/fhttp"
+	"fortio.org/fortio/fnet"
 	"fortio.org/fortio/jrpc"
 	"fortio.org/fortio/periodic"
 	"fortio.org/fortio/stats"
@@ -42,6 +43,7 @@ const (
 	RestRunURI    = "rest/run"
 	RestStatusURI = "rest/status"
 	RestStopURI   = "rest/stop"
+	RestDNS       = "rest/dns"
 	ModeGRPC      = "grpc"
 )
 
@@ -109,6 +111,13 @@ type StatusReply struct {
 	Statuses StatusMap
 }
 
+type DNSReply struct {
+	jrpc.ServerReply
+	Name string
+	IPv4 []string
+	IPv6 []string
+}
+
 // Error writes serialized ServerReply marked as error, to the writer.
 func Error(w http.ResponseWriter, msg string, err error) {
 	if w == nil {
@@ -185,7 +194,7 @@ func FormValue(r *http.Request, json map[string]interface{}, key string) string
 // RESTRunHandler is api version of UI submit handler.
 // TODO: refactor common option/args/flag parsing between uihandler.go and this.
 func RESTRunHandler(w http.ResponseWriter, r *http.Request) { //nolint:funlen
-	log.LogRequest(r, "REST Run Api call")
+	log.LogRequest(r, "REST Run call")
 	w.Header().Set("Content-Type", "application/json")
 	data, err := io.ReadAll(r.Body) // must be done before calling FormValue
 	if err != nil {
@@ -215,6 +224,7 @@ func RESTRunHandler(w http.ResponseWriter, r *http.Request) { //nolint:funlen
 	log.Infof("Starting API run %s load request from %v for %s", runner, r.RemoteAddr, url)
 	async := (FormValue(r, jd, "async") == "on")
 	payload := FormValue(r, jd, "payload")
+	methodOverride := FormValue(r, jd, "X")
 	labels := FormValue(r, jd, "labels")
 	resolution, _ := strconv.ParseFloat(FormValue(r, jd, "r"), 64)
 	percList, _ := stats.ParsePercentiles(FormValue(r, jd, "p"))
@@ -222,6 +232,7 @@ func RESTRunHandler(w http.ResponseWriter, r *http.Request) { //nolint:funlen
 	durStr := FormValue(r, jd, "t")
 	jitter := (FormValue(r, jd, "jitter") == "on")
 	uniform := (FormValue(r, jd, "uniform") == "on")
+	logErrors := (FormValue(r, jd, "log-errors") == "on")
 	nocatchup := (FormValue(r, jd, "nocatchup") == "on")
 	stdClient := (FormValue(r, jd, "stdclient") == "on")
 	h2 := (FormValue(r, jd, "h2") == "on")
@@ -242,7 +253,7 @@ func RESTRunHandler(w http.ResponseWriter, r *http.Request) { //nolint:funlen
 		}
 	}
 	c, _ := strconv.Atoi(FormValue(r, jd, "c"))
-	out := io.Writer(os.Stderr)
+	out := io.Writer(os.Stdout)
 	if len(percList) == 0 && !strings.Contains(r.URL.RawQuery, "p=") {
 		percList = DefaultPercentileList
 	}
@@ -276,6 +287,8 @@ func RESTRunHandler(w http.ResponseWriter, r *http.Request) { //nolint:funlen
 	httpopts.Insecure = httpsInsecure
 	httpopts.Resolve = resolve
 	httpopts.H2 = h2
+	httpopts.LogErrors = logErrors
+	httpopts.MethodOverride = methodOverride
 	// Set the connection reuse range.
 	err = bincommon.ConnectionReuseRange.
 		WithValidator(bincommon.ConnectionReuseRangeValidator(httpopts)).
@@ -318,8 +331,11 @@ func RESTRunHandler(w http.ResponseWriter, r *http.Request) { //nolint:funlen
 		}
 	}
 	fhttp.OnBehalfOf(httpopts, r)
+	// Needed to reply the id, will be reused in Normalize() later as already set
+	// but we also do it early even for sync case so that the id is available to save JSON
+	// in case of init error.
+	ro.GenID()
 	if async {
-		ro.GenID() // Needed to reply the id, will be reused in Normalize() later as already set
 		reply := AsyncReply{RunID: runid, Count: 1, ResultID: ro.ID, ResultURL: ID2URL(r, ro.ID)}
 		reply.Message = "started" //nolint:goconst
 		err := jrpc.ReplyOk(w, &reply)
@@ -400,6 +416,18 @@ func Run(w http.ResponseWriter, r *http.Request, jd map[string]interface{},
 		aborter.StartChan <- false
 		log.LogVf("REST run %d really done - after channel write", ro.RunID)
 	}()
+	savedAs := ""
+	jsonData, jerr := json.MarshalIndent(res, "", "  ")
+	if jerr != nil {
+		log.Fatalf("Unable to json serialize result: %v", jerr) //nolint:gocritic // gocritic doesn't know fortio's log.Fatalf does pani
+	}
+	jsonStr := string(jsonData)
+	log.LogVf("Serialized to %s", jsonStr)
+	id := res.Result().ID
+	doSave := (FormValue(r, jd, "save") == "on")
+	if doSave && id != "" {
+		savedAs = SaveJSON(id, jsonData)
+	}
 	if err != nil {
 		log.Errf("Init error for %s mode with url %s and options %+v : %v", runner, url, ro, err)
 		if !htmlMode {
@@ -407,36 +435,24 @@ func Run(w http.ResponseWriter, r *http.Request, jd map[string]interface{},
 		}
 		return res, "", nil, err
 	}
-	json, err := json.MarshalIndent(res, "", "  ")
-	if err != nil {
-		log.Fatalf("Unable to json serialize result: %v", err) //nolint:gocritic // gocritic doesn't know fortio's log.Fatalf does pani
-	}
-	jsonStr := string(json)
-	log.LogVf("Serialized to %s", jsonStr)
-	id := res.Result().ID
-	doSave := (FormValue(r, jd, "save") == "on")
-	savedAs := ""
-	if doSave {
-		savedAs = SaveJSON(id, json)
-	}
 	if w == nil {
 		// async or html but nil w (no json output): no result to output
-		return res, savedAs, json, nil
+		return res, savedAs, jsonData, nil
 	}
 	if htmlMode {
 		// Already set in api mode but not in html mode
 		w.Header().Set("Content-Type", "application/json")
 	}
-	_, err = w.Write(json)
+	_, err = w.Write(jsonData)
 	if err != nil {
 		log.Errf("Unable to write json output for %v: %v", r.RemoteAddr, err)
 	}
-	return res, savedAs, json, nil
+	return res, savedAs, jsonData, nil
 }
 
 // RESTStatusHandler will print the state of the runs.
 func RESTStatusHandler(w http.ResponseWriter, r *http.Request) {
-	log.LogRequest(r, "REST Status Api call")
+	log.LogRequest(r, "REST Status call")
 	runid, _ := strconv.ParseInt(r.FormValue("runid"), 10, 64)
 	statusReply := StatusReply{}
 	if runid != 0 {
@@ -455,12 +471,12 @@ func RESTStatusHandler(w http.ResponseWriter, r *http.Request) {
 
 // RESTStopHandler is the api to stop a given run by runid or all the runs if unspecified/0.
 func RESTStopHandler(w http.ResponseWriter, r *http.Request) {
-	log.LogRequest(r, "REST Stop Api call")
-	w.Header().Set("Content-Type", "application/json")
+	log.LogRequest(r, "REST Stop call")
 	runid, _ := strconv.ParseInt(r.FormValue("runid"), 10, 64)
 	waitStr := strings.ToLower(r.FormValue("wait"))
 	wait := (waitStr != "" && waitStr != "off" && waitStr != "false")
 	i, rid := StopByRunID(runid, wait)
+	log.Debugf("REST Stop completed, stopped %d runs, rid %s", i, rid)
 	reply := AsyncReply{RunID: runid, Count: i, ResultID: rid, ResultURL: ID2URL(r, rid)}
 	if wait && i == 1 {
 		reply.Message = StateStopped.String()
@@ -499,6 +515,7 @@ func StopByRunID(runid int64, wait bool) (int, string) {
 	}
 	// else: Stop one
 	v, found := runs[runid]
+	log.Debugf("Interrupting runid %d, found %v : %+v", runid, found, v)
 	if !found {
 		uiRunMapMutex.Unlock()
 		log.Infof("Runid %d not found to interrupt", runid)
@@ -516,8 +533,8 @@ func StopByRunID(runid int64, wait bool) (int, string) {
 	v.aborter.Abort(wait)
 	if wait {
 		log.LogVf("REST stop, wait requested, reading additional channel signal")
-		<-v.aborter.StartChan
-		log.LogVf("REST stop, received all done signal")
+		b := <-v.aborter.StartChan
+		log.LogVf("REST stop, received all done signal got %v", b)
 	}
 	log.LogVf("Returning from Abort %d call with wait %v", runid, wait)
 	return 1, rid
@@ -531,6 +548,31 @@ func RemoveRun(id int64) {
 	log.LogVf("REST Removed run %d", id)
 }
 
+func RESTDNSHandler(w http.ResponseWriter, r *http.Request) {
+	log.LogRequest(r, "REST DNS call")
+	name := r.FormValue("name")
+	ips, err := fnet.ResolveAll(r.Context(), name, "ip")
+	if err != nil {
+		err = jrpc.ReplyError(w, "dns failed", err)
+		if err != nil {
+			log.Errf("Error replying: %v", err)
+		}
+		return
+	}
+	reply := DNSReply{Name: name, IPv4: make([]string, 0), IPv6: make([]string, 0)}
+	for _, ip := range ips {
+		if ip.To4() == nil {
+			reply.IPv6 = append(reply.IPv6, ip.String())
+		} else {
+			reply.IPv4 = append(reply.IPv4, ip.String())
+		}
+	}
+	err = jrpc.ReplyOk(w, &reply)
+	if err != nil {
+		log.Errf("Error replying: %v", err)
+	}
+}
+
 // AddHandlers adds the REST Api handlers for run, status and stop.
 // uiPath must end with a /.
 func AddHandlers(ahook bincommon.FortioHook, mux *http.ServeMux, baseurl, uiPath, datadir string) {
@@ -542,7 +584,9 @@ func AddHandlers(ahook bincommon.FortioHook, mux *http.ServeMux, baseurl, uiPath
 	mux.HandleFunc(restStatusPath, RESTStatusHandler)
 	restStopPath := uiPath + RestStopURI
 	mux.HandleFunc(restStopPath, RESTStopHandler)
-	log.Printf("REST API on %s, %s, %s", restRunPath, restStatusPath, restStopPath)
+	dnsPath := uiPath + RestDNS
+	mux.HandleFunc(dnsPath, RESTDNSHandler)
+	log.Printf("REST API on %s, %s, %s, %s", restRunPath, restStatusPath, restStopPath, dnsPath)
 }
 
 // SaveJSON save Json bytes to give file name (.json) in data-path dir.
@@ -594,7 +638,7 @@ func UpdateRun(ro *periodic.RunnerOptions) *periodic.Aborter {
 
 func GetRun(id int64) *Status {
 	uiRunMapMutex.Lock()
-	res := runs[id]
+	res := runs[id] // TODO: race: need to copy because the pointers to options can be changed by the runners
 	uiRunMapMutex.Unlock()
 	return res
 }
@@ -612,6 +656,16 @@ func GetAllRuns() StatusMap {
 	return res
 }
 
+// RunMetrics returns the number of currently running runs as well as the total runs so far.
+// acquire the lock once (per scrape) for all stats.
+func RunMetrics() (int, int64) {
+	uiRunMapMutex.Lock()
+	runid := id
+	n := len(runs)
+	uiRunMapMutex.Unlock()
+	return n, runid
+}
+
 func SetDataDir(datadir string) {
 	dataDir = datadir
 }
