@@ -1,4 +1,4 @@
-// Copyright 2017 Istio Authors
+// Copyright 2017 Fortio Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -202,14 +202,15 @@ type HistogramData struct {
 // NewHistogram creates a new histogram (sets up the buckets).
 // Divider value can not be zero, otherwise returns zero.
 func NewHistogram(offset float64, divider float64) *Histogram {
-	h := new(Histogram)
-	h.Offset = offset
 	if divider == 0 {
 		return nil
 	}
-	h.Divider = divider
-	h.Hdata = make([]int32, numBuckets)
-	return h
+	h := Histogram{
+		Offset:  offset,
+		Divider: divider,
+		Hdata:   make([]int32, numBuckets),
+	}
+	return &h
 }
 
 // Val2Bucket values are kept in two different structure
@@ -270,18 +271,25 @@ func (h *Histogram) RecordN(v float64, n int) {
 
 // Records v value to count times.
 func (h *Histogram) record(v float64, count int) {
-	// Scaled value to bucketize - we subtract epsilon because the interval
+	// Scaled value to bucketize - we used to subtract epsilon because the interval
 	// is open to the left ] start, end ] so when exactly on start it has
-	// to fall on the previous bucket. TODO add boundary tests
-	scaledVal := (v-h.Offset)/h.Divider - 0.0001
+	// to fall on the previous bucket: which is more correctly done using
+	// math.Ceil()-1 but that doesn't work... so back to epsilon distance check
+	scaledVal := (v - h.Offset) / h.Divider
 	var idx int
 	if scaledVal <= firstValue {
 		idx = 0
 	} else if scaledVal > lastValue {
 		idx = numBuckets - 1 // last bucket is for > last value
 	} else {
-		// else we look it up
-		idx = lookUpIdx(int(scaledVal))
+		// else we look it up (with the open interval adjustment)
+		svInt := int(scaledVal)
+		delta := scaledVal - float64(svInt)
+		if delta < 1e-12 {
+			svInt--
+		}
+		log.Debugf("v %f -> scaledVal %.17f ceil %f delta %g - svInt %d", v, scaledVal, math.Ceil(scaledVal), delta, svInt)
+		idx = lookUpIdx(svInt)
 	}
 	h.Hdata[idx] += int32(count)
 }
